package com.sample;

import java.util.concurrent.atomic.AtomicInteger;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;

/**
 * This is a sample class to launch a rule.
 */
public class DroolsTest {
	
	public static final void main(String[] args) {
		// load up the knowledge base
		KnowledgeBase kbase = readKnowledgeBase(args[0]);
		final int size = Integer.parseInt(args[1]);
		StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
		// go !
		long time = System.currentTimeMillis();
		AtomicInteger count = new AtomicInteger();
		ksession.setGlobal("count", count);
		for(int i = 0; i < size; i++) {
			ksession.insert(new A(i));
		}
		for(int i = 0; i < size; i+=2) {
			ksession.insert(new B(i));
		}
		ksession.fireAllRules();
		System.out.println(count);
		System.out.println((System.currentTimeMillis() - time) + "ms");
	}

	private static KnowledgeBase readKnowledgeBase(String name) {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource(name), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	public abstract static class X {

		private final int id;

		public int getId() {
			return id;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + id;
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			X other = (X) obj;
			if (id != other.id)
				return false;
			return true;
		}

		public X(int id) {
			super();
			this.id = id;
		}
		
	}
	
	public static class A extends X {
		public A(int id) { super(id); }
	}
	
	public static class B extends X {
		public B(int id) { super(id); }
	}

}